
#include "crypto_onetimeauth_poly1305.h"

#define crypto_onetimeauth_poly1305_implementation_name \
    crypto_onetimeauth_poly1305_53_implementation_name

#define crypto_onetimeauth crypto_onetimeauth_poly1305_53
#define crypto_onetimeauth_verify crypto_onetimeauth_poly1305_53_verify
